package cn.minbb.edu.activities;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.design.widget.TabLayout;
import android.support.v4.view.ViewPager;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.edu.R;
import cn.minbb.edu.activities.ui.main.SectionsPagerAdapter;
import cn.minbb.edu.app.URL;
import cn.minbb.edu.data.ResponseResult;
import cn.minbb.edu.model.Course;
import cn.minbb.edu.util.HttpRequestUtil;

public class CourseCenterActivity extends AppCompatActivity {

    @BindView(R.id.tabs)
    TabLayout tabs;
    @BindView(R.id.refresh)
    SwipeRefreshLayout refresh;
    @BindView(R.id.view_pager)
    ViewPager viewPager;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_course_center);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadCourseData();
    }

    public static void start(Context context) {
        Intent starter = new Intent(context, CourseCenterActivity.class);
        context.startActivity(starter);
    }

    private void initView() {
    }

    private void setListener() {
        refresh.setOnRefreshListener(this::onRefresh);
    }

    private void loadCourseData() {
        refresh.setRefreshing(true);
        // 加载选定课程标签
        new HttpRequestUtil(URL.request(URL.URLEnum.COURSE_HOT), new HttpRequestUtil.RequestCallback() {
            @Override
            public void failed(String s) {
                runOnUiThread(() -> {
                    refresh.setRefreshing(false);
                    Toast.makeText(getApplicationContext(), "请求失败\n" + s, Toast.LENGTH_SHORT).show();
                });
            }

            @Override
            public void success(String s) {
                runOnUiThread(() -> {
                    refresh.setRefreshing(false);
                    ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                    Collection collection = result.getDataset();
                    List<String> tabTitleList = Arrays.asList("政治", "历史", "语文", "数学", "英语");
                    for (Object o : collection) {
                        Course course = JSONObject.parseObject(JSONObject.toJSONString(o), Course.class);
                    }
//                    SectionsPagerAdapter sectionsPagerAdapter = new SectionsPagerAdapter(
//                            getSupportFragmentManager(), tabTitleList);
//                    viewPager.setAdapter(sectionsPagerAdapter);
//                    tabs.setupWithViewPager(viewPager);
                });
            }
        }).post(null);
    }

    public void onRefresh() {
        loadCourseData();
    }

    public void onViewClick(View view) {
        switch (view.getId()) {
            case R.id.fab:
                Snackbar.make(view, "AHaHa", Snackbar.LENGTH_LONG).setAction("OK", null).show();
                break;
        }
    }
}