package cn.minbb.edu.activities;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.BottomNavigationView;
import android.support.design.widget.TabLayout;
import android.support.v4.view.ViewPager;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.AppCompatDelegate;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;
import com.bumptech.glide.Glide;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.bingoogolapple.bgabanner.BGABanner;
import cn.bingoogolapple.bgabanner.transformer.RotatePageTransformer;
import cn.bingoogolapple.bgabanner.transformer.TransitionEffect;
import cn.minbb.edu.R;
import cn.minbb.edu.activities.ui.main.SectionsPagerAdapter;
import cn.minbb.edu.adapter.MainActivityBGABannerAdapter;
import cn.minbb.edu.adapter.MainActivityPagerAdapter;
import cn.minbb.edu.adapter.MainBasicCourseAdapter;
import cn.minbb.edu.adapter.MainHotCourseAdapter;
import cn.minbb.edu.app.URL;
import cn.minbb.edu.ctrl.UserManager;
import cn.minbb.edu.data.BannerData;
import cn.minbb.edu.data.ResponseResult;
import cn.minbb.edu.model.Banner;
import cn.minbb.edu.model.Course;
import cn.minbb.edu.model.Subject;
import cn.minbb.edu.model.User;
import cn.minbb.edu.util.HttpRequestUtil;

public class MainActivity extends AppCompatActivity {

    @BindView(R.id.viewPager)
    ViewPager viewPager;
    @BindView(R.id.navigation)
    BottomNavigationView navigation;

    private List<View> views = new ArrayList<>(3);
    private BGABanner mainBanner;
    private SwipeRefreshLayout mainRefreshLayout, courseRefreshLayout;
    private TextView courseHotTip, courseExcellentTip;
    private TabLayout courseTabs;
    private ViewPager courseViewPager;
    private ImageView avatar;
    private TextView name, school;

    private User user;
    private BannerData bannerData;
    private MainHotCourseAdapter courseHotAdapter, courseExcellentAdapter;
    private List<MainHotCourseAdapter.ItemBean> courseHotList = new ArrayList<>(), courseExcellentList = new ArrayList<>();
    private List<MainBasicCourseAdapter.ItemBean> courseBasicList = new ArrayList<>();
    private int mainTaskCount = 0;

    static {
        AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onResume() {
        super.onResume();
        UserManager.userAutoLogin(new UserManager.AutoLoginCallback() {
            @Override
            public void success() {
                runOnUiThread(() -> loadUserData());
            }

            @Override
            public void failed() {
                runOnUiThread(() -> Toast.makeText(getApplicationContext(), "自动登录失败", Toast.LENGTH_SHORT).show());
            }
        });
    }

    public static void start(Context context) {
        Intent starter = new Intent(context, MainActivity.class);
        context.startActivity(starter);
    }

    private void initView() {
        // 初始化 Page 页面
        LayoutInflater layoutInflater = LayoutInflater.from(this);
        View page_1, page_2, page_3;
        page_1 = layoutInflater.inflate(R.layout.page_1, null);
        page_2 = layoutInflater.inflate(R.layout.page_2, null);
        page_3 = layoutInflater.inflate(R.layout.page_3, null);
        views.add(page_1);
        views.add(page_2);
        views.add(page_3);
        viewPager.setAdapter(new MainActivityPagerAdapter(views));
        // 主页搜索
        page_1.findViewById(R.id.search).setOnClickListener(view -> SearchActivity.start(MainActivity.this));
        // 主页轮播
        mainBanner = page_1.findViewById(R.id.mainBanner);
        mainBanner.setTransitionEffect(TransitionEffect.Default);       // 用 Java 代码方式设置切换动画
        mainBanner.setPageTransformer(new RotatePageTransformer());
        mainBanner.setPageChangeDuration(3000);                         // 设置 page 切换时长
        mainBanner.setAdapter(new MainActivityBGABannerAdapter(MainActivity.this));
        // 主页 热门课程和优秀课程
        RecyclerView courseHotView = page_1.findViewById(R.id.courseHotView);
        courseHotTip = page_1.findViewById(R.id.courseHotTip);
        RecyclerView courseExcellentView = page_1.findViewById(R.id.courseExcellentView);
        courseExcellentTip = page_1.findViewById(R.id.courseExcellentTip);
        // 主页 所有课程
        courseHotView.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false));
        courseHotAdapter = new MainHotCourseAdapter(this, courseHotList);
        courseHotAdapter.setOnItemClickListener(position -> {
            MainHotCourseAdapter.ItemBean itemBean = courseHotList.get(position);
            CourseDetailActivity.start(MainActivity.this, itemBean.getCourseId());
        });
        courseHotView.setAdapter(courseHotAdapter);
        courseExcellentView.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false));
        courseExcellentAdapter = new MainHotCourseAdapter(this, courseExcellentList);
        courseExcellentAdapter.setOnItemClickListener(position -> {
            MainHotCourseAdapter.ItemBean itemBean = courseExcellentList.get(position);
            CourseDetailActivity.start(MainActivity.this, itemBean.getCourseId());
        });
        courseExcellentView.setAdapter(courseExcellentAdapter);

        // 课程中心
        courseTabs = page_2.findViewById(R.id.tabs);
        courseViewPager = page_2.findViewById(R.id.course_view_pager);
        // 首页刷新视图
        mainRefreshLayout = page_1.findViewById(R.id.mainRefreshLayout);
        // 课程广场页面刷新视图
        courseRefreshLayout = page_2.findViewById(R.id.course_refresh_layout);
        // 用户信息
        avatar = page_3.findViewById(R.id.avatar);
        name = page_3.findViewById(R.id.name);
        school = page_3.findViewById(R.id.school);
    }

    private void setListener() {
        // Page 滑动监听
        viewPager.addOnPageChangeListener(new ViewPager.OnPageChangeListener() {
            @Override
            public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {
            }

            @Override
            public void onPageSelected(int position) {
                switch (position) {
                    case 0:
                        navigation.setSelectedItemId(R.id.navigation_1);
                        break;
                    case 1:
                        navigation.setSelectedItemId(R.id.navigation_2);
                        break;
                    case 2:
                        navigation.setSelectedItemId(R.id.navigation_3);
                        break;
                }
            }

            @Override
            public void onPageScrollStateChanged(int state) {
            }
        });
        // 底部导航监听
        navigation.setOnNavigationItemSelectedListener(item -> {
            switch (item.getItemId()) {
                case R.id.navigation_1:
                    viewPager.setCurrentItem(0);
                    return true;
                case R.id.navigation_2:
                    viewPager.setCurrentItem(1);
                    return true;
                case R.id.navigation_3:
                    viewPager.setCurrentItem(2);
                    return true;
            }
            return false;
        });
        // 刷新监听
        mainRefreshLayout.setOnRefreshListener(() -> {
            // 更新数据
            mainTaskCount = 0;
            loadMainData();
        });
        courseRefreshLayout.setOnRefreshListener(this::loadCourseData);
        // 轮播监听
        mainBanner.setDelegate((banner, itemView, model, position) -> BannerActivity.start(this, bannerData.getLinks().get(position)));
    }

    private void loadData() {
        loadUserData();
        loadMainData();
        loadCourseData();
    }

    private void loadUserData() {
        // 用户相关
        user = UserManager.getCurrentUser();
        if (user == null) {
            LoginActivity.start(this);
            MainActivity.this.finish();
        }
        String nameText = user.getName();
        if (TextUtils.isEmpty(nameText)) {
            name.setText("请设置姓名");
        } else {
            name.setText(nameText);
        }
        school.setText(user.getSchool());
        String avatar = user.getAvatar();
        if (!TextUtils.isEmpty(avatar)) {
            Glide.with(this).load(avatar)
                    .placeholder(R.drawable.banner_loading_16by9)
                    .error(R.drawable.banner_loading_error_16by9)
                    .centerCrop().dontAnimate().into(this.avatar);
        }
    }

    private void loadMainData() {
        // 加载主页轮播
        new HttpRequestUtil(URL.request(URL.URLEnum.BANNERS), new HttpRequestUtil.RequestCallback() {
            @Override
            public void failed(String s) {
                runOnUiThread(() -> {
                    mainTaskCount++;
                    Toast.makeText(getApplicationContext(), "请求加载主页轮播失败\n" + s, Toast.LENGTH_SHORT).show();
                });
            }

            @Override
            public void success(String s) {
                runOnUiThread(() -> {
                    mainTaskCount++;
                    if (mainTaskCount == 3) mainRefreshLayout.setRefreshing(false);
                    ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                    Collection collection = result.getDataset();
                    List<String> models = new ArrayList<>(collection.size());
                    List<String> tips = new ArrayList<>(collection.size());
                    List<String> links = new ArrayList<>(collection.size());
                    for (Object o : collection) {
                        Banner banner = JSONObject.parseObject(JSONObject.toJSONString(o), Banner.class);
                        models.add(banner.getCover());
                        tips.add(banner.getTitle());
                        links.add(banner.getPath());
                    }
                    bannerData = new BannerData(models, tips, links);
                    mainBanner.setData(models, tips);
                });
            }
        }).get(null);
        // 加载热门课程
        new HttpRequestUtil(URL.request(URL.URLEnum.COURSE_HOT), new HttpRequestUtil.RequestCallback() {
            @Override
            public void failed(String s) {
                runOnUiThread(() -> {
                    mainTaskCount++;
                    Toast.makeText(getApplicationContext(), "请求加载热门课程失败\n" + s, Toast.LENGTH_SHORT).show();
                });
            }

            @Override
            public void success(String s) {
                runOnUiThread(() -> {
                    mainTaskCount++;
                    if (mainTaskCount == 3) mainRefreshLayout.setRefreshing(false);
                    ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                    courseHotTip.setText(String.format("+ %s more", result.getMessage()));
                    Collection collection = result.getDataset();
                    courseHotList.clear();
                    for (Object o : collection) {
                        Course course = JSONObject.parseObject(JSONObject.toJSONString(o), Course.class);
                        courseHotList.add(new MainHotCourseAdapter.ItemBean(course.getId(), course.getCover(), course.getName(), course.getIntroduction()));
                    }
                    courseHotAdapter.notifyDataSetChanged();
                });
            }
        }).post(null);
        // 加载优秀课程
        new HttpRequestUtil(URL.request(URL.URLEnum.COURSE_EXCELLENT), new HttpRequestUtil.RequestCallback() {
            @Override
            public void failed(String s) {
                runOnUiThread(() -> {
                    mainTaskCount++;
                    Toast.makeText(getApplicationContext(), "请求加载优秀课程失败\n" + s, Toast.LENGTH_SHORT).show();
                });
            }

            @Override
            public void success(String s) {
                runOnUiThread(() -> {
                    mainTaskCount++;
                    if (mainTaskCount == 3) mainRefreshLayout.setRefreshing(false);
                    ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                    courseExcellentTip.setText(String.format("+ %s more", result.getMessage()));
                    Collection collection = result.getDataset();
                    courseExcellentList.clear();
                    for (Object o : collection) {
                        Course course = JSONObject.parseObject(JSONObject.toJSONString(o), Course.class);
                        courseExcellentList.add(new MainHotCourseAdapter.ItemBean(course.getId(), course.getCover(), course.getName(), course.getIntroduction()));
                    }
                    courseExcellentAdapter.notifyDataSetChanged();
                });
            }
        }).post(null);
    }

    private void loadCourseData() {
        // 加载课程中心
        new HttpRequestUtil(URL.request(URL.URLEnum.SUBJECT_USER), new HttpRequestUtil.RequestCallback() {
            @Override
            public void failed(String s) {
                runOnUiThread(() -> {
                    courseRefreshLayout.setRefreshing(false);
                    Toast.makeText(getApplicationContext(), "请求失败\n" + s, Toast.LENGTH_SHORT).show();
                });
            }

            @Override
            public void success(String s) {
                runOnUiThread(() -> {
                    courseRefreshLayout.setRefreshing(false);
                    courseBasicList.clear();
                    ResponseResult result = JSONObject.parseObject(s, ResponseResult.class);
                    Collection collection = result.getDataset();
                    List<String> tabTitleList = new ArrayList<>(collection.size());
                    List<Integer> tabIdList = new ArrayList<>(collection.size());
                    for (Object o : collection) {
                        Subject subject = JSONObject.parseObject(JSONObject.toJSONString(o), Subject.class);
                        tabIdList.add(subject.getId());
                        tabTitleList.add(subject.getName());
                    }
                    SectionsPagerAdapter sectionsPagerAdapter = new SectionsPagerAdapter(
                            getSupportFragmentManager(), tabTitleList, tabIdList);
                    courseViewPager.setAdapter(sectionsPagerAdapter);
                    courseTabs.setupWithViewPager(courseViewPager);
                });
            }
        }).post(user);
    }

    public void onViewClick(View view) {
        switch (view.getId()) {
            case R.id.edit:
                UserInfoActivity.start(this);
                break;
            case R.id.course:
                MyCourseActivity.start(MainActivity.this);
                break;
            case R.id.settings:
                SettingsActivity.start(this);
                break;
            case R.id.help:
                HelpActivity.start(this);
                break;
            case R.id.exit:
                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setTitle("退出登录？").setMessage("确定要退出登录吗？")
                        .setNegativeButton("取消", (dialog, which) -> {
                        })
                        .setPositiveButton("退出", (dialog, which) -> {
                            UserManager.userLogout(user);
                            LoginActivity.start(this);
                            MainActivity.this.finish();
                        })
                        .create().show();
                break;
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(0, 1, 0, "搜索")
                .setIcon(R.drawable.ic_search_white_24dp)
                .setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        menu.add(0, 2, 0, "通知")
                .setIcon(R.drawable.ic_notifications_white_24dp)
                .setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case 1:
                Toast.makeText(getApplicationContext(), "搜索", Toast.LENGTH_SHORT).show();
                break;
            case 2:
                Toast.makeText(getApplicationContext(), "通知", Toast.LENGTH_SHORT).show();
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle("退出？")
                    .setMessage("确定要退出吗？")
                    .setNegativeButton("取消", (dialog, which) -> {
                    })
                    .setPositiveButton("退出", (dialog, which) -> {
                        MainActivity.this.finish();
                    })
                    .create()
                    .show();
        }
        return super.onKeyDown(keyCode, event);
    }
}
