package cn.minbb.edu.adapter;

import android.content.Context;
import android.support.annotation.NonNull;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.resource.drawable.GlideDrawable;
import com.bumptech.glide.request.RequestListener;
import com.bumptech.glide.request.target.Target;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.edu.R;
import lombok.Getter;

public class MainHotCourseAdapter extends RecyclerView.Adapter<MainHotCourseAdapter.ViewHolder> {

    // 映射数据
    private Context context;
    private List<ItemBean> itemBeen;
    private LayoutInflater layoutInflater;

    private ItemClickListener itemClickListener;

    public interface ItemClickListener {
        void onItemClick(int position);
    }

    public void setOnItemClickListener(ItemClickListener itemClickListener) {
        this.itemClickListener = itemClickListener;
    }

    public MainHotCourseAdapter(Context context, List<ItemBean> itemBeen) {
        this.context = context;
        this.itemBeen = itemBeen;
        layoutInflater = LayoutInflater.from(context);
    }

    @NonNull
    @Override
    public MainHotCourseAdapter.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        // 转换一个 View 布局，决定了 item 的样子， 参数：1.上下文 2.Xml 布局资源 3.为 null
        View view = layoutInflater.inflate(R.layout.item_hot_course, parent, false);
        // 创建一个 StaggerViewHolder 对象并把 ViewHolder 对象传出去
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull MainHotCourseAdapter.ViewHolder holder, int position) {
        Glide.with(context)
                .load(itemBeen.get(position).courseImage)
                .placeholder(R.drawable.banner_loading_16by9)
                .error(R.drawable.banner_loading_error_16by9)
                .listener(new RequestListener<String, GlideDrawable>() {
                    @Override
                    public boolean onException(Exception e, String model, Target<GlideDrawable> target, boolean isFirstResource) {
                        return false;
                    }

                    @Override
                    public boolean onResourceReady(GlideDrawable resource, String model, Target<GlideDrawable> target, boolean isFromMemoryCache, boolean isFirstResource) {
                        return false;
                    }
                })
                .centerCrop()
                .dontAnimate()
                .into(holder.courseImage);
        holder.courseName.setText(itemBeen.get(position).courseName);
        holder.courseTip.setText(itemBeen.get(position).courseTip);
        if (null != itemClickListener) {
            holder.itemView.setOnClickListener(view -> itemClickListener.onItemClick(holder.getAdapterPosition()));
        }
    }

    @Override
    public int getItemCount() {
        return itemBeen.size();
    }

    // ViewHolder 用于缓存控件
    static class ViewHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.cardView)
        View cardView;
        @BindView(R.id.courseImage)
        ImageView courseImage;
        @BindView(R.id.courseName)
        TextView courseName;
        @BindView(R.id.courseTip)
        TextView courseTip;

        ViewHolder(View view) {
            super(view);
            ButterKnife.bind(this, view);
        }
    }

    public static class ItemBean {

        @Getter
        Integer courseId;
        String courseImage;
        String courseName;
        String courseTip;

        public ItemBean(Integer courseId, String courseImage, String courseName, String courseTip) {
            this.courseId = courseId;
            this.courseImage = courseImage;
            this.courseName = courseName;
            this.courseTip = courseTip;
        }
    }
}
