package cn.minbb.edu.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.TextView;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.edu.R;
import cn.minbb.edu.model.Stage;
import lombok.Getter;
import lombok.Setter;

public class StageAdapter extends BaseAdapter {

    private List<ItemBean> itemBeen;
    private LayoutInflater layoutInflater;

    public StageAdapter(Context context, List<ItemBean> itemBeen) {
        layoutInflater = LayoutInflater.from(context);
        this.itemBeen = itemBeen;
    }

    @Override
    public int getCount() {
        // 获取数据量
        return itemBeen.size();
    }

    @Override
    public Object getItem(int position) {
        // 获取对应 ID 项对应的 Item
        return itemBeen.get(position);
    }

    @Override
    public long getItemId(int position) {
        // 获取对应项 ID
        return position;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ViewHolder viewHolder;
        if (convertView == null) {
            // 将 XML 转化为 View ，并不涉及到具体的布局，所以第二个参数通常设置为 null
            convertView = layoutInflater.inflate(R.layout.item_stage, parent, false);
            viewHolder = new ViewHolder(convertView);
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }
        // 取出 ItemBean 对象
        ItemBean itemBean = itemBeen.get(position);
        // 设置控件数据
        viewHolder.stage.setText(itemBean.stage.getName());
        return convertView;
    }

    public static class ItemBean {

        @Getter
        @Setter
        private Stage stage;

        public ItemBean(Stage stage) {
            this.stage = stage;
        }
    }

    class ViewHolder {
        @BindView(R.id.stage)
        TextView stage;

        ViewHolder(View view) {
            ButterKnife.bind(this, view);
        }
    }
}
