package cn.minbb.edu.util;

import android.util.Log;

import com.alibaba.fastjson.JSONObject;

import java.io.IOException;
import java.util.Map;

import cn.minbb.edu.app.URL;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import okhttp3.ResponseBody;

public class HttpRequestUtil {

    private static String TAG = "HttpRequestUtil";

    private String url;
    private RequestCallback requestCallback;

    public HttpRequestUtil(String url, RequestCallback requestCallback) {
        this.url = url;
        this.requestCallback = requestCallback;
    }

    public void get(Map<String, String> params) {
        StringBuilder stringBuilder = new StringBuilder();
        if (null != params) {
            stringBuilder.append("?");
            for (Map.Entry<String, String> entry : params.entrySet()) {
                stringBuilder.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
            }
        }
        OkHttpClient client = new OkHttpClient();
        Request request = new Request.Builder().url(url + stringBuilder).get().build();
        Log.d(TAG, "请求地址=" + request.url());
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                if (null != requestCallback) requestCallback.failed("请求失败 " + e.getMessage());
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                if (response.isSuccessful()) {
                    ResponseBody responseBody = response.body();
                    if (null != responseBody) {
                        String data = responseBody.string();
                        Log.d(TAG, "收到数据=" + data);
                        if (null != requestCallback) requestCallback.success(data);
                    } else {
                        if (null != requestCallback) requestCallback.failed("数据为空");
                    }
                } else {
                    if (null != requestCallback) requestCallback.failed("isNotSuccessful");
                }
            }
        });
    }

    public void post(Object param) {
        String requestMessage = null == param ? "" : JSONObject.toJSONString(param);
        Log.d(TAG, "请求报文 = " + requestMessage);
        OkHttpClient client = new OkHttpClient();
        Request request = new Request.Builder()
                .header("Accept", "*/*")
                .addHeader("Connection", "close")
                .addHeader("MultipleDevicesAuth", "true")
                .addHeader("Content-Type", "application/json; charset=UTF-8")
                .url(url)
                .post(RequestBody.create(MediaType.parse("application/json; charset=UTF-8"), requestMessage))
                .build();
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                e.printStackTrace();
                Log.e(TAG, e.getMessage() + "");
                if (null != requestCallback) requestCallback.failed("请求失败 " + e.getMessage());
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                if (response.isSuccessful()) {
                    ResponseBody responseBody = response.body();
                    if (null != responseBody) {
                        try {
                            String data = responseBody.string();
                            Log.d(TAG, "收到数据 = " + data);
                            if (null != requestCallback) requestCallback.success(data);
                        } catch (IOException e) {
                            e.printStackTrace();
                            if (null != requestCallback) requestCallback.failed("\\n not found");
                        }
                    } else {
                        if (null != requestCallback) requestCallback.failed("收到数据为空");
                    }
                } else {
                    if (null != requestCallback) requestCallback.failed("isNotSuccessful");
                }
            }
        });
    }

    public interface RequestCallback {
        void failed(String s);

        void success(String s);
    }
}
